
      subroutine fix_higgs_width_scheme(higgs_width_scheme_in)
      implicit none
      integer higgs_width_scheme_in,higgs_width_scheme
      common /hwscom/ higgs_width_scheme
      
      higgs_width_scheme=higgs_width_scheme_in
      end
c-------------------------------------------------------------
      double precision function BW_factor(QQQ,mh_nom)
      implicit none
      double precision QQQ,mh_nom,Gamma_ratio,GammaRatio,res,rr
      integer higgs_width_scheme
      common /hwscom/ higgs_width_scheme
      double precision deltaWidth
      common /DeltaWidth/ deltaWidth
      
      rr=QQQ/mh_nom
      Gamma_ratio=GammaRatio(QQQ)
      if (deltaWidth.eq.0d0) then !if Zero Width Approximation
            res=1d0
      else
            if (higgs_width_scheme.eq.0) then 
            !
            !     Default scheme      
            !
            !     Q*Gamma(Q)
            !     ----------
            !     Mh*Gamma(Mh)
            !
                  res=rr*Gamma_ratio
            
            elseif (higgs_width_scheme.eq.1) then 
            !     
            !    Seymour scheme :  
            !
            !     Q*Gamma(Q)        d^2         +       (1 - Q^2 / Mh^2)^2
            !     ----------  * -------------------------------------------
            !     Mh*Gamma(Mh)      d^2 * Q^4/Mh^4   + (1 - Q^2 / Mh^2)^2
            !
            res=rr*Gamma_ratio / rr**4 * (deltaWidth**2+(1d0-rr**2)**2) 
     $              /          ((1d0-rr**2)**2+deltaWidth**2 * rr**4)
            else
                  print*,"ERROR in fix_higgs_width_scheme: "
                  print*,"unrecognized Higgs width scheme = ",
     $                  higgs_width_scheme
                  print*,"valid values = 0,1"
                  stop
            endif
      endif !-- if Zero Width Approximation
      
      
      BW_factor=res
      end
      

c-------------------------------------------------------------
      subroutine read_grid(gridfilename)
      implicit none
      character*40 gridfilename
      !
      integer Npvv
      parameter (Npvv=16100)
      double precision Q(Npvv),Gtot(Npvv),BRph(Npvv)
      double precision BRww(Npvv),BRzz(Npvv),BRbb(Npvv)
      common /GTOTBRcom/ Q,Gtot,BRph,BRww,BRzz,BRbb
      !
      integer IO,irec,i
      double precision Qx,Gt,ph,ww,zz,bb
      !
      integer maxipos
      common /maxiposcom/ maxipos
      !
      open(unit=14,file=gridfilename,status='old')
      read(14,*, IOSTAT = IO ) 
      read(14,*, IOSTAT = IO )
      read(14,*, IOSTAT = IO )
      irec=1
      do i=1,16100
            read(14,*, IOSTAT = IO ) Qx,Gt,ph,ww,zz,bb
            if (IO.ge.0) then 
                Q(irec)=Qx
                Gtot(irec)=Gt
                BRph(irec)=ph
                BRww(irec)=ww
                BRzz(irec)=zz
                BRbb(irec)=bb
                irec=irec+1
            else
                  GOTO 897
            endif
      enddo
      
  897 close(14)
      
      maxipos=irec
      
      print*,"Reading of Higgs width and BRs file"
      print*,"Number of points in the grid :",maxipos
      !stop
      end

c-------------------------------------------------------------
C----- Sets total width at mh_nominal (Gtotc) and deltaWidth
      subroutine fitbr(QQQc,higgs_width)
      implicit none
      double precision QQQc,higgs_width
      double precision  Gtotc
      common /GammaTotalcom/ Gtotc
      double precision deltaWidth
      common /DeltaWidth/ deltaWidth  

      !
      integer Npvv
      parameter (Npvv=16100)
      double precision Q(Npvv),Gtot(Npvv),BRph(Npvv)
      double precision BRww(Npvv),BRzz(Npvv),BRbb(Npvv)
      common /GTOTBRcom/ Q,Gtot,BRph,BRww,BRzz,BRbb
      !
      integer DecayMode
      common /decaymodecom/ DecayMode
      !

      integer iposition   ,find_position
      !
      double precision A
      
      !
      if (DecayMode.eq.-1) then 
            Gtotc=0d0
            deltaWidth=0d0
      else
               
            if (higgs_width.eq.0d0) then 
                  iposition=find_position(QQQc)
                  A = (Q(iposition+1)-QQQc)
     &                   /(Q(iposition+1)-Q(iposition))
                  Gtotc=Gtot(iposition+1)*(1d0-A)+Gtot(iposition)*A
                  deltaWidth = Gtotc/QQQc
            else
                  Gtotc=higgs_width
                  deltaWidth = Gtotc/QQQc
            endif
      endif
      write(*,"(A,(f16.4))"),"higgs width = ",Gtotc
      !print*,"between ",Gtot(iposition),Gtot(iposition+1)
      !stop
      end
c-------------------------------------------------------------
      double precision function my_interpolate(F,myQ,QQc)
      implicit none
      integer Npvv
      parameter (Npvv=16100)
      double precision F(Npvv),myQ(Npvv)
      integer ipos,find_position
      double precision res,QQc,A
      
      
      ipos=find_position(QQc)
      A=(myQ(ipos+1)-QQc)
     &                   /(myQ(ipos+1)-myQ(ipos))
      res=F(ipos+1)*(1d0-A)+F(ipos)*A
      my_interpolate=res
      end 
      
c-------------------------------------------------------------
C----- Returns BR at QQQ
      double precision function BranchingRatio(QQQ)
      implicit none
      double precision QQQ
      !
      integer Npvv
      parameter (Npvv=16100)
      double precision Q(Npvv),Gtot(Npvv),BRph(Npvv)
      double precision BRww(Npvv),BRzz(Npvv),BRbb(Npvv)
      common /GTOTBRcom/ Q,Gtot,BRph,BRww,BRzz,BRbb
      !
      double precision ris
      integer iposition ,find_position     
      !
      integer DecayMode
      common /decaymodecom/ DecayMode
      !--
      double precision A
      
      if (DecayMode.gt.0) then 
            iposition=find_position(QQQ)
            A = (Q(iposition+1)-QQQ)/(Q(iposition+1)-Q(iposition))
            select case(DecayMode)
            case(1)!gamma gamma
            ris=BRph(iposition+1)*(1d0-A)+BRph(iposition)*A
            case(2)!WW
            ris=BRww(iposition+1)*(1d0-A)+BRww(iposition)*A
            case(3)!ZZ
            ris=BRzz(iposition+1)*(1d0-A)+BRzz(iposition)*A
            case(4)!bb
            ris=BRbb(iposition+1)*(1d0-A)+BRbb(iposition)*A
            end select
      else
            ris=1
      endif
      BranchingRatio=ris
      end

c-------------------------------------------------------------
C----- Returns 
      double precision function GammaRatio(QQQc)
      implicit none
      double precision QQQc,Gloc,ris
      double precision  Gtotc
      common /GammaTotalcom/ Gtotc
      !
      integer Npvv
      parameter (Npvv=16100)
      double precision Q(Npvv),Gtot(Npvv),BRph(Npvv)
      double precision BRww(Npvv),BRzz(Npvv),BRbb(Npvv)
      common /GTOTBRcom/ Q,Gtot,BRph,BRww,BRzz,BRbb
      !
      integer DecayMode
      common /decaymodecom/ DecayMode
      integer iposition ,find_position
      !----------------
      double precision A,my_interpolate
      
      if (DecayMode.GT.-1) then   
            iposition=find_position(QQQc)
            A = (Q(iposition+1)-QQQc)/(Q(iposition+1)-Q(iposition))
            Gloc=Gtot(iposition+1)*(1d0-A)+Gtot(iposition)*A
            
            ris=Gloc/Gtotc
c            print*,QQQc,"Gtot=",Gtotc," Gloc=",Gloc," ratio=",ris
      !---- end of DecayMode>-1
      else ! => DecayMode=-1
        ris=1d0
      endif  
      GammaRatio =ris
      end
c-------------------------------------------------------------

      integer function find_position(QQQ)
      implicit none
      double precision QQQ
      !
      integer Npvv
      parameter (Npvv=16100)
      double precision Q(Npvv),Gtot(Npvv),BRph(Npvv)
      double precision BRww(Npvv),BRzz(Npvv),BRbb(Npvv)
      common /GTOTBRcom/ Q,Gtot,BRph,BRww,BRzz,BRbb
      !
      integer maxipos
      common /maxiposcom/ maxipos
      !
      integer i,j,k,maxbisect
      i=1   
      j=maxipos
      DO  maxbisect=1,1000
            k=(i+j)/2   
            IF (QQQ < Q(k)) THEN 
                  j=k  
            ELSE
                  i=k
            END IF
            IF (i+1 >= j) EXIT
      END DO
c      print*,"found position : ",QQQ," is between ",
c     $      Q(i)," and ",Q(i+1)
      find_position=i
      end
